EmoteNonsense = {}

--*********
--Init:
--Initialises variables from SavedVariables, or if they aren't present, sets defaults.
--*********

local enAddonFrame = CreateFrame("FRAME");
enAddonFrame:RegisterEvent("ADDON_LOADED");
enAddonFrame:RegisterEvent("PLAYER_LOGIN");

enAddonFrame:SetScript("OnEvent", function(self, event, name)
    if event == "ADDON_LOADED" and name == "EmoteNonsense" then
        if enTextColour == nil then
            -- Load Defaults
            enTextColour = "FFFFFF" -- Hex Code for text between quotation marks.
            enEmoteColour = "FF8040" --  Hex Code for text between asterisks.
            enPlayerColour = "00CCFF"  --  Hex Code for text between asterisks
        end
        if enDoFlashClient == nil then
            enDoFlashClient = true
            enDoRaidWarning = true
        end
        if enNames == nil then
            enNames = {}
        end
    end
end)

--*********
--EmoteNonsense_NoCase
--@Remarks      - Takes an input, returning a value that can be read case-insensitive
--@Param(text)  - String you wish to check.
--@Return       - Formatted lower, upper within square brackets.
--*********
function EmoteNonsense_NoCase(text)
    text = string.gsub(text, "%a", function (f)
        return string.format("[%s%s]", string.lower(f), string.upper(f)) end)
    return text
end

--*********
--EmoteNonsense_IsValidHex
--@Remarks          - Determines if a provided string is a valid hex value by converting the string to a Base 16 Number
--@Param(colour)    - String you wish to check.
--@Return           - True if valid, false if invalid.
--*********
local function EmoteNonsense_IsValidHex(colour)
    local colour = tonumber(colour, 16)
    if colour == nil then
        return false
    else
        return true
    end
end

--*********
--EmoteNonsense_DoFlashClient
--@Remarks  - Flashes the WoW Icon if enDoFlashClient is True.
--*********
local function EmoteNonsense_DoFlashClient()
    if enDoFlashClient == true then
        FlashClientIcon()
    end
end

--*********
--EmoteNonsense_DoRaidWarning
--@Remarks          - Creates a "Raid Warning" style popup if your name is mentioned, if enDoRaidWarning is True.
--@Param(sender)    - A string containing the name of the player who mentioned your name.
--*********
local function EmoteNonsense_DoRaidWarning(sender)
    if enDoRaidWarning == true then
        RaidNotice_AddMessage(RaidWarningFrame, sender.." has mentioned you!", ChatTypeInfo["RAID_WARNING"])
		PlaySoundFile("Sound\\Doodad\\BellTollNightElf.ogg")
    end
end

--*********
--EmoteNonsense_SubTextInsideEmotes
--@Remarks          - Substitutes text between quotation marks with enTextColour's value.
--@Param(msg)       - A string containing the message portion of the event.
--@Param(sender)    - A string containing the sender portion of the event.
--@Funcs            - EmoteNonsense_CheckForNames
--@Return           - Returns false so the message isn't removed, a modified 'msg' with colours, and the rest of the original event.
--*********
local function EmoteNonsense_SubTextInsideEmotes(self, event, msg, sender, ...)
    if msg ~= nil then
        msg = EmoteNonsense_CheckForNames(msg, sender)
    end
    return false, gsub(msg,("%b\"\""),"|cFF"..enTextColour.."%1|r"), sender, ...
end

--*********
--EmoteNonsense_SubEmotesInsideText
--@Remarks          - Substitutes text between asterisks with enEmoteColour's value.
--@Param(msg)       - A string containing the message portion of the event.
--@Param(sender)    - A string containing the sender portion of the event.
--@Funcs            - EmoteNonsense_CheckForNames
--@Return           - Returns false so the message isn't removed, a modified 'msg' with colours, and the rest of the original event.
--*********
local function EmoteNonsense_SubEmotesInsideText(self, event, msg, sender, ...)
    if msg ~= nil then
        msg = EmoteNonsense_CheckForNames(msg, sender)
    end
    return false, gsub(msg,("%b**"),"|cFF"..enEmoteColour.."%1|r"), sender, ...
end

--*********
--EmoteNonsense_CheckForNames
--@Remarks          - Checks for the players character name, as well as selected names from enNames, and colours them as required.
--@Param(msg)       - A string containing the message portion of the event.
--@Param(sender)    - A string containing the sender portion of the event.
--@Funcs            - EmoteNonsense_DoRaidWarning, EmoteNonsense_DoFlashClient
--@Return           - Returns a modified msg with highlighted player name colours. If name is not found, returns an unmodified message.
--*********
function EmoteNonsense_CheckForNames(msg, sender)
    local enWasFound = 0;
    local enPlayerName, _ = UnitName("player")
    if enNames ~= nil then
        for i=1,#enNames do
            local currentName = enNames[i]
            if msg:find("%f[%a]" .. EmoteNonsense_NoCase(currentName) .. "%f[%A]") then
                EmoteNonsense_DoRaidWarning(sender)
                EmoteNonsense_DoFlashClient()
                enWasFound = 1;
                msg = "|cFF"..enPlayerColour..">>|r"..msg
            end
        end
    end
    if enWasFound == 0 then
        if msg:find("%f[%a]" .. EmoteNonsense_NoCase(enPlayerName) .. "%f[%A]") then
            EmoteNonsense_DoRaidWarning(sender)
            EmoteNonsense_DoFlashClient()
            msg = "|cFF"..enPlayerColour..">>|r"..msg
       end
    end
    return msg
end

--*********
--SlashCmdList["EN"]
--@Remarks          - Handles all the slash commands.
--@Param(msg)       - A string containing the message received from the chatbox.
--*********
SLASH_EN1 = "/en"
SlashCmdList["EN"] = function(msg)
    local command, arg = msg:match("^(%S*)%s*(.-)$");
    if command == "text" and arg ~= "" then
        if (EmoteNonsense_IsValidHex(arg) == false or #arg ~= 6) then
            print("Colour must be hex & 6 chars, ie FFFFFF")
        else
            enTextColour = arg
            print ("Text colour changed to:|cFF"..enTextColour.." "..enTextColour.."|r")
        end
    elseif command == "emote" and arg ~= "" then
        if (EmoteNonsense_IsValidHex(arg) == false or #arg ~= 6) then
            print("Colour must be hex & 6 chars, ie FFFFFF")
        else
            enEmoteColour = arg
            print ("Emote colour changed to:|cFF"..enEmoteColour.." "..enEmoteColour.."|r")
        end
    elseif command == "player" and arg ~= "" then
        if (EmoteNonsense_IsValidHex(arg) == false or #arg ~= 6) then
            print("Colour must be hex & 6 chars, ie FFFFFF")
        else
            enPlayerColour = arg
            print ("Player colour changed to:|cFF"..enPlayerColour.." "..enPlayerColour.."|r")
        end
    elseif command == "name" and arg ~= "" then
        if enNames[arg] then
            print("You've already added that name! If you'd like to remove it, please use /en remove {name}")
        else
            table.insert(enNames, strlower(arg))
            print("Added the name "..arg)
        end
    elseif command == "remove" and arg ~= "" then
        for i=1,#enNames do
            if enNames[i] == strlower(arg) then
                table.remove(enNames, i)
            end
        end
        print("Removed all traces of the name "..arg)
    elseif command == "flash" then
        if enDoFlashClient == true then
            enDoFlashClient = false
            print ("Client Flash Disabled")
        elseif enDoFlashClient == false then
            enDoFlashClient = true
            print ("Client Flash Enabled")
        end
    elseif command == "warning" then
        if enDoRaidWarning == true then
            enDoRaidWarning = false
            print ("Raid Warning Disabled")
        elseif enDoRaidWarning == false then
            enDoRaidWarning = true
            print ("Raid Warning Enabled")
        end
		
--*********
--To play or not play sound.
--*********
	elseif command == "sound" then
        if enDoPlaySound == true then
            enDoPlaySound = false
            print ("Sound Disabled")
        elseif enDoPlaySound == false then
            enDoPlaySound = true
            print ("Sound Enabled")
		end
		
--*********
--Command list
--*********	
    else
		print("EmoteNonsense Help List")
		print("/en text {hexcode} to change text color")
		print("/en emote {hexcode} to change emote color")
		print("/en player {hexcode} to change player's (your) color")
		print("/en name (name) to add names (no parenthases)")
		print("/en remove (name) to remove names (no parenthases)")
		print("/en warning to disable raid warnings")
		print("/en sound to disable addon sounds")
    end
end

--*********
--EmoteNonsense_MessageFilter
--@Remarks          - Uses WoW's AddMessageEventFilter with EmoteNonsense_SubTextInsideEmotes and EmoteNonsense_SubEmotesInsideText to colour the messages.
--@Funcs            - EmoteNonsense_SubTextInsideEmotes, EmoteNonsense_SubEmotesInsideText
--*********
function EmoteNonsense_MessageFilter()
    ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE", EmoteNonsense_SubTextInsideEmotes)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", EmoteNonsense_SubEmotesInsideText)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", EmoteNonsense_SubEmotesInsideText)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", EmoteNonsense_SubEmotesInsideText)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", EmoteNonsense_SubEmotesInsideText)
    ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", EmoteNonsense_SubEmotesInsideText)
end

EmoteNonsense_MessageFilter()